function expBatching(loss,data,method,maxIter,param1,param2,param3)
randn('state',0);
rand('state',0);

debug = 0;

if isdeployed
    debug = 0;
    maxIter = str2double(maxIter);
    param1 = str2double(param1);
    param2 = str2double(param2);
    param3 = str2double(param3);
    warning off all
end


fprintf('loss = %s, data = %s, method = %s\nmaxIter = %d, param1 = %e, param2 = %e, param3 = %e\n',loss,data,method,maxIter,param1,param2,param3);

resultDir = sprintf('batching/results/%s',loss);
resultFile = sprintf('%s/%s_%s_%d_%e_%e_%e.mat',resultDir,data,method,maxIter,param1,param2,param3);
resultFile = fixResultFile(resultFile);

if ~exist(resultDir,'dir')
    mkdir(resultDir);
end

if ~debug && exist(resultFile,'file')
    fprintf('Result file exists\n');
    return;
end

useSparse = 0;
useScaled = 0;
fprintf('Loading Data and setting up objective...');
switch loss
    case {'logreg','logregl2'}
        switch data
            case 'test'
                X = randn(100,100);
                w = randn(100,1);
                y = sign(X*w + randn(100,1));
            case 'sido'
                load sido0_train
                y = load('sido0_train.targets');
            case 'thrombin'
                [y,X] = libsvmread(sprintf('%s/data/classificationBinary/thrombin.txt',pwd));
            case 'spam'
                load trec2005.mat
                X = A';
                clear A
                y(y==0) = -1;
            case 'rcv1'
                [y,X] = libsvmread(sprintf('%s/data/classificationBinary/rcv1_train.binary','/data/schmidtm'));
            case 'real-sim'
                [y,X] = libsvmread(sprintf('%s/data/classificationBinary/real-sim.svml',pwd));
            case 'news'
                [y,X] = libsvmread(sprintf('%s/data/classificationBinary/news20.binary','/data/schmidtm'));
            case 'rcv12'
                fprintf('(training set)...');
                [yt,Xt] = libsvmread(sprintf('%s/data/classificationBinary/rcv1_train.binary','/data/schmidtm'));
                fprintf('(testing set)...');
                [y,X] = libsvmread(sprintf('%s/data/classificationBinary/rcv1_test.binary','/data/schmidtm'));
                fprintf('(combining y)...');
                y = [yt;y];
                fprintf('(combining X)...');
                X = [Xt;X];   
        end
        X = [ones(size(X,1),1) X];
        %X = full(X);
        [nInstances,nVars] = size(X);
        w = zeros(nVars,1);
        Xt = X';
        switch loss
            case 'logreg'
                funObj = @(w)LogisticLoss(w,X,y);
                funObjBatch = @(w,batch)LogisticLossIncremental_Xtranspose(w,Xt,y,batch);
                
                if issparse(X)
                    useSparse = 1;
                end
            case 'logregl2'
                lambda = 1e-2;
                funObj = @(w)penalizedL2(w,@LogisticLoss,lambda,X,y);
                funObjBatch = @(w,batch)penalizedL2(w,@LogisticLossIncremental_Xtranspose,lambda*length(batch)/nInstances,Xt,y,batch);
                funObjBatch2 = @(w,batch)penalizedL2(w,@LogisticLossIncremental_Xtranspose,lambda,Xt,y,batch);
                funObjBatch_scaled = @(w,c,batch)LogisticLossIncremental_Xtranspose_scaled(w,c,Xt,y,batch);
                
                if issparse(X)
                    useScaled = 1;
                end
                options.strongly = 1;
        end
    case {'mlogreg'}
        switch data
            case 'wave'
                X = load('uci.waveform40.data');
                y = X(:,end)+1;
                X = [ones(size(X,1),1) standardizeCols(X(:,1:end-1))];
            case 'shuttle'
                X = load('statlog.shuttle.data');
                y = X(:,end);
                X = [ones(size(X,1),1) standardizeCols(X(:,1:end-1))];
            case 'usps'
                load('usps.mat');
                X = sparse([ones(11000,1) X/256]);
                y = ones(11000,1);
                for i = 1:10
                    y(1 + (i-1)*1100:i*1100) = i;
                end
            case 'mnist16'
                load('mnist16.mat');
                X = [train1;test1
                    train2;test2
                    train3;test3
                    train4;test4
                    train5;test5
                    train6;test6
                    train7;test7
                    train8;test8
                    train9;test9
                    train0;test0];
                X = sparse([ones(70000,1) X/256]);
                y = [ones(length(train1),1);ones(length(test1),1)
                    2*ones(length(train2),1);2*ones(length(test2),1)
                    3*ones(length(train3),1);3*ones(length(test3),1)
                    4*ones(length(train4),1);4*ones(length(test4),1)
                    5*ones(length(train5),1);5*ones(length(test5),1)
                    6*ones(length(train6),1);6*ones(length(test6),1)
                    7*ones(length(train7),1);7*ones(length(test7),1)
                    8*ones(length(train8),1);8*ones(length(test8),1)
                    9*ones(length(train9),1);9*ones(length(test9),1)
                    10*ones(length(train0),1);10*ones(length(test0),1)];
            case 'mnist'
                load('mnist.mat');
                X = double([train1;test1
                    train2;test2
                    train3;test3
                    train4;test4
                    train5;test5
                    train6;test6
                    train7;test7
                    train8;test8
                    train9;test9
                    train0;test0]);
                X = sparse([ones(70000,1) X/256]);
                y = [ones(length(train1),1);ones(length(test1),1)
                    2*ones(length(train2),1);2*ones(length(test2),1)
                    3*ones(length(train3),1);3*ones(length(test3),1)
                    4*ones(length(train4),1);4*ones(length(test4),1)
                    5*ones(length(train5),1);5*ones(length(test5),1)
                    6*ones(length(train6),1);6*ones(length(test6),1)
                    7*ones(length(train7),1);7*ones(length(test7),1)
                    8*ones(length(train8),1);8*ones(length(test8),1)
                    9*ones(length(train9),1);9*ones(length(test9),1)
                    10*ones(length(train0),1);10* ...
                     ones(length(test0),1)];
            case 'rcv1'
                [y,X] = libsvmread(sprintf('%s/data/classificationMulti/rcv1_train.multiclass','/data/schmidtm'));
			case 'rcv12'
                fprintf('(training set)...');
                [yt,Xt] = libsvmread(sprintf('%s/data/classificationMulti/rcv1_train.multiclass','/data/schmidtm'));
                fprintf('(testing set)...');
                [y,X] = libsvmread(sprintf('%s/data/classificationMulti/rcv1_test.multiclass','/data/schmidtm'));
                fprintf('(combining y)...');
                y = [yt;y];
                fprintf('(combining X)...');
                X = [Xt;X];   
        end
        [nInstances,nVars] = size(X);
        nClasses = max(y);
        w = zeros(nVars*nClasses,1);
        Xt = X';
        
        lambda = 1;
        funObj = @(w)penalizedL2(w,@SoftmaxLoss,lambda,X,y,nClasses);
        funObjBatch = @(w,batch)penalizedL2(w,@SoftmaxLossIncremental_Xtranspose,lambda*length(batch)/nInstances,Xt,y,nClasses,batch);
        
        if issparse(X)
            %useScaled = 1;
        end
        options.strongly = 1;
    case 'crf'
        lambda = 1;
        load coNLL_train.mat
        if strcmp(data,'coNLL2')
            Xtrain = X;
            ytrain = y;
            load coNLL_test.mat
            X = [Xtrain;X];
            y = [ytrain;y];
		end
        nFeatures = max(X);
        nStates = max(y);
        [w,v_start,v_end,v] = crfChain_initWeights(nFeatures,nStates);
        featureStart = cumsum([1 nFeatures(1:end)]);
        sentences = crfChain_initSentences(y);
        nInstances = size(sentences,1);
        maxSentenceLength = 1+max(sentences(:,2)-sentences(:,1));
        w = [w(:);v_start(:);v_end(:);v(:)];
        funObj = @(w)crfChain_lossL2C(w,X,y,nStates,nFeatures,featureStart,sentences,maxSentenceLength,lambda);
        funObjBatch = @(w,batch)crfChain_lossL2C(w,X,y,nStates,nFeatures,featureStart,sentences(batch,:),maxSentenceLength,lambda*length(batch)/nInstances);
        options.strongly = 1;
    case {'crf2d','crf2d_trbp'}
        lambda = 1;
        switch data
            case 'noisyX'
                nInstances = 100;
                [Xnode,Xedge,Y,edgeStruct,infoStruct] = prepareNoisyX(nInstances);
            case 'usps'
                [Xnode,Xedge,Y,edgeStruct,infoStruct] = prepareNoisyUSPS;
                nInstances = size(Y,1);
            case 'mnist'
                [Xnode,Xedge,Y,edgeStruct,infoStruct] = prepareNoisyMNIST;
                nInstances = size(Y,1);
			case 'kumar'
                [Xnode,Xedge,Y,edgeStruct,nodeMap,edgeMap,nRows,nCols] = prepareBinaryDenoising;
                nInstances = size(Y,1);
		end
        w = UGM_initWeights(nodeMap,edgeMap);
		switch loss
			case 'crf2d'
				funObj = @(w)penalizedL2(w,@UGM_CRF_PseudoNLL,lambda,Xnode,Xedge,Y,nodeMap,edgeMap,edgeStruct);
				funObjBatch = @(w,batch)penalizedL2(w,@UGM_CRF_PseudoNLL,lambda*length(batch)/nInstances,Xnode(batch,:,:),Xedge(batch,:,:),Y(batch,:,:),nodeMap,edgeMap,edgeStruct);
			case 'crf2d_trbp'
				treeWeights = (1/2)*ones(edgeStruct.nEdges,1); % We use two trees: all horizontal edges and all vertical edges
				funObj = @(w)penalizedL2(w,@UGM_CRFLoss,lambda,Xnode,Xedge,Y,edgeStruct,infoStruct,@UGM_Infer_TRBP,treeWeights);
				funObjBatch = @(w,batch)penalizedL2(w,@UGM_CRFLoss,lambda*length(batch)/nInstances,Xnode(batch,:,:),Xedge(batch,:,:),Y(batch,:,:),edgeStruct,infoStruct,@UGM_Infer_TRBP,treeWeights);
		end
		options.strongly = 1;
    case 'dwi'
        % Tristan's code
        % grid
        model.x = 0:10:1000;
        model.z = [0:10:1000]';
        % freguencies
        ft = [0:.5:125]; nft=length(ft);
        If = [11    13    14    24    29    32    36    48];
        model.freq = ft(If);
        % peak frequency of source signature
        model.f0   = 10;
        % spatial width of point source
        model.sigma = 10;
        % source coordinates, vertical line at x=100
        model.xsrc = 100;
        model.zsrc = linspace(0,1000,101);
        % receiver coordinates, vertical line at x=900
        model.xrec = 900;
        model.zrec = linspace(0,1000,101);
        
        % true velocity, constant background with some features.
        [xx,zz] = meshgrid(model.x,model.z);
        m = 0*xx(:) + .25;
        m(find(abs(xx-300)<20&abs(zz-300)<50)) = .2;
        m(find(abs(xx-700)<50&abs(zz-500)<20)) = .3;
        m(find(((xx-500).^2+(zz-800).^2)<4000)) = .2;
        m(find(((xx-500).^2+(zz-800).^2)<2000)) = .25;
        
        % initial velociy
        m0 = 0*m + .25;
        
        % data generation
        tic
        fprintf('Generating Data\n');
        Dobs = F(m,model);
        nInstances = length(model.zsrc)
        E  = eye(nInstances);
        toc
        
        w = m0;
        funObj = @(m0)Jls(m0,Dobs,E,model);
        funObjBatch = @(m0,batch)Jls(m0,Dobs,E(:,batch),model);
end
fprintf('done\n');

switch method
    case 'lbfgs'
        options.maxIter = maxIter;
        options.maxFunEvals = maxIter;
        options.Display = 'Full';
        options.Corr = 10;
        [w,f,exitflag,output] = minFunc(funObj,w,options);
        fVal = output.trace.fval;
        fEval = output.trace.funcCount*nInstances;
		optCond = output.trace.optCond;
        batchSizes = nInstances;
    case {'sgd1','sgd2'} % Constant step size
        maxSGDiter = nInstances*maxIter;
        theta = param1;
        fEval = zeros(0,1);
        fVal = zeros(0,1);
		optCond = zeros(0,1);
        
        if useScaled
            c = 1;
            s = (1-2*theta*lambda/nInstances);
        end
        
        for iter = 1:maxSGDiter
            i = ceil(rand*nInstances);
            
            if useScaled
                [f,g] = funObjBatch_scaled(w,c,i);
            else
                [f,g] = funObjBatch(w,i);
            end
            
            switch method
                case 'sgd1'
                    stepSize = theta;
                case 'sgd2'
                    stepSize = theta/iter;
                    if useScaled
                        s = (1-2*stepSize*lambda/nInstances);
                    end
            end
            
            if useScaled
                sparseOps_densePlusSparse_inPlace(w,(-stepSize/(s*c))*g);
                c = s*c;
            elseif useSparse
                sparseOps_densePlusSparse_inPlace(w,-stepSize*g);
            else
                w = w - stepSize*g;
            end
            
            if mod(iter,round(nInstances*.05)) == 0
                if useScaled
                    w = c*w;
                    c = 1;
                end
                
                fEval(end+1,1) = iter;
                [fFull gFull] = funObj(w);
				fVal(end+1) = fFull;
				optCond(end+1,1) = max(abs(gFull));
                
                fprintf('Iter %d of %d, fsub = %e, f = %e\n',iter,maxSGDiter,f,fVal(end));
                
                if isnan(fVal(end,1)) & isnan(f) & isnan(g)
                    fprintf('Gone crazy, stopping\n');
                    break;
                end
            end
        end
        batchSizes = 1;
    case 'batching'
        options.trace = 1;
        options.maxIter = maxIter;
        options.t0 = param1;
        options.gamma = param2;
        options.delta = param3;
        [w,output] = batchingLBFGS(funObjBatch,w,nInstances,options);
        fVal = output.fVal;
        fEval = output.fEval;
		optCond = output.optCond;
        batchSizes = output.bSizes;
    case 'batching2'
        options.trace = 1;
        options.maxIter = maxIter;
        options.t0 = param1;
        options.gamma = 1.1;
        options.delta = 1;
		options.scaleStep = param2;
		options.scaleObj = param3;
        [w,output] = batchingLBFGS(funObjBatch,w,nInstances,options);
        fVal = output.fVal;
        fEval = output.fEval;
        batchSizes = output.bSizes;
	case 'batchingWolfe'
        options.trace = 1;
        options.maxIter = maxIter;
        options.t0 = param1;
        options.gamma = param2;
        options.delta = param3;
		%options.strongly = 1;
		options.wolfe = 1;
        [w,output] = batchingLBFGS(funObjBatch,w,nInstances,options);
        fVal = output.fVal;
        fEval = output.fEval;
		optCond = output.optCond;
        batchSizes = output.bSizes;
end

if debug
    pause
elseif strcmp(loss,'crf2d')
    save(resultFile,'fVal','fEval','nInstances','w','batchSizes','optCond');
else
    save(resultFile,'fVal','fEval','nInstances','batchSizes','optCond');
end