function op = opLagInterp2(x1,y1,x2,y2)

op = opFunction(length(x2)*length(y2),length(y1)*length(x1),{@(z)lagintp2(x1,y1,z,x2,y2,[],1),@(z)lagintp2(x1,y1,[],x2,y2,z,-1)},1);


function out = lagintp2(x,y,s,x1,y1,s1,flag)

% 2D Cubic Lagrange interpolation 
% use:		output = lagintp(x, y, s, x1, y1, s1, flag)
% input:	x,y - grid
%           s - corresponding data, for forward mode
%           x1,y1 - grid
%           s1 - corresponding data, for adjoint mode
%           flag - 1:forward, 2:adjoint
%output: 	forward: s1, adjoint: s
%note:		all vectors are assumed column vectors


if flag>0
    s   = reshape(s,length(x),length(y));
    tmp = lagintp1(x,s,x1,[],1);
    out = transpose(lagintp1(y,transpose(tmp),y1,[],1));
    out = out(:);
else
    s1  = reshape(s1,length(x1),length(y1));
    tmp = transpose(lagintp1(y,[],y1,transpose(s1),-1));
    out = lagintp1(x,[],x1,tmp,-1);
    out = out(:);
end

function out = lagintp1(x,y,x1,y1,flag)

% 1D Cubic Lagrange interpolation 
% use:		output = lagintp(x, s, x1, s1, flag)
% input:	x - grid
%           s - corresponding data, for forward mode
%           x1 - grid
%           s1 - corresponding data, for adjoint mode
%           flag - 1:forward, 2:adjoint
%output: 	forward: s1, adjoint: s
%note:		all vectors are assumed column vectors

dx = x(2)-x(1);
ik = find(x1 >= x(1) & x1 <= x(end));
n=length(x);n1=length(x1);nk=length(ik);
A=spalloc(n1,n,4*nk);

a=1;b=2;c=3;d=4;
if ~length(ik)
    out = [];
    return;
end

for i=1:nk
    k=ik(i);
    if x1(k)<x(b)
        while (x1(k)<x(b))&(b-1>1)
            b=b-1;
        end
        a=b-1;c=b+1;d=c+1;
    elseif x1(k)>x(c)
        while (x1(k)>x(c))&(c+1<n)
           c=c+1;
        end
        a=c-2;b=c-1;d=c+1;
    end
     A(k,a) = ((x1(k)-x(b))*(x1(k)-x(c))*(x1(k)-x(d)))/((x(a)-x(b))*(x(a)-x(c))*(x(a)-x(d)));
     A(k,b) = ((x1(k)-x(a))*(x1(k)-x(c))*(x1(k)-x(d)))/((x(b)-x(a))*(x(b)-x(c))*(x(b)-x(d)));
     A(k,c) = ((x1(k)-x(b))*(x1(k)-x(a))*(x1(k)-x(d)))/((x(c)-x(b))*(x(c)-x(a))*(x(c)-x(d)));
     A(k,d) = ((x1(k)-x(b))*(x1(k)-x(c))*(x1(k)-x(a)))/((x(d)-x(b))*(x(d)-x(c))*(x(d)-x(a)));
end
if flag>0
    out=A*y;
    out(1:ik(1)-1)=0;
    out(ik(end)+1:n1)=0;
else
    out=A'*y1; 
end
