function [fc,gc] = Jls(m,D,W,model)
%% least-squares misfit for `waveform inversion'
%% ||(D - PH[m]^{-1}Q)W||_F^2
%% 
%% m       - (1/velocity)^2 [s^2/km^2] as vector, grid defined by model.z,model.x
%% model.x - vector with x-grid e.g., 0:10:1000
%% model.z - vector with z-grid e.g., 0:10:1000
%%
%% D       - Data. Each column is a frequency slice. Frequencies are given by model.freq
%%           Each frequency slice is a vectorized source-receiver matrix, sizes determined by 
%%	     model.xsrc,model.zsrc,model.xrec,model.zrec
%% model.xsrc, model.zsrc - source grid
%% model.xrec, model.zrec - receiver grid
%%
%% W       - Each source-receiver matrix is multiplied by W. Can be used for randomized source 
%%           superposition or single source selection
%%
%% model.sigma - determines spatial extent of point source
%% model.f0    - determines peak frequency of source signature 

nsrc = length(model.xsrc)*length(model.zsrc);
nrec = length(model.xrec)*length(model.zrec);
nx = length(model.x);
nz = length(model.z);
freq = model.freq; nfreq = length(freq);

%%
P = opLagInterp2(model.z,model.x,model.zrec,model.xrec);
Q = Src(model);
w = freq.^2.*exp(-(freq/model.f0).^2);

%spmd
    fc = 0;
    gc = 0*m;
    for k = 1:nfreq
        modelf = model;
        modelf.freq = freq(k);
        Df = reshape(D(:,k),nrec,nsrc)*W;
        
        %%%% forward modelling
        U   = w(k)*Linv(m,Q*W,1,modelf);
        Dtf = P*U;
        
        
        %%%% misfit
        Rf   = Dtf - Df;
        fc   = fc + .5*norm(Rf(:)).^2;
    
        %%% gradient
        V = Linv(m,P'*Rf,-1,modelf);
        gc = gc + (2*pi*1e-3*freq(k))^2*real(sum(conj(U).*V,2));
    end
%end

% f = 0;
% g = 0*m;
% for k = 1:length(fc)
%     f = f + fc{k};
%     g = g + gc{k};
% end
