function output = F(m,model)
%% m       - (1/velocity)^2 [s^2/km^2] as vector, grid defined by model.z,model.x
%% model.x - vector with x-grid e.g., 0:10:1000
%% model.z - vector with z-grid e.g., 0:10:1000
%%
%% model.sigma - determines spatial extent of point source
%% model.f0    - determines peak frequency of source signature
%%
%% output      - Data. Each column is a frequency slice. Frequencies are given by model.freq
%%               Each frequency slice is a vectorized source-receiver matrix, sizes determined by
%%               model.xsrc,model.zsrc,model.xrec,model.zrec
%% model.freq  - frequencies [Hz]
%% model.xsrc, model.zsrc - source grid
%% model.xrec, model.zrec - receiver grid

nsrc = length(model.xsrc)*length(model.zsrc);
nrec = length(model.xrec)*length(model.zrec);
nx = length(model.x);
nz = length(model.z);
freq = model.freq; nfreq = length(freq);

%% SPOT, etc.
P = opLagInterp2(model.z,model.x,model.zrec,model.xrec);
Q = Src(model);
w = freq.^2.*exp(-(freq/model.f0).^2);

%spmd
    output = zeros(nsrc*nrec,nfreq);
    for k = 1:nfreq
        fprintf('Generating data, k = %d of %d\n',k,nfreq);
       modelf = model; modelf.freq = freq(k);
       U = w(k)*Linv(m,Q,1,modelf);
       tmp = P*U;
       output(:,k) = tmp(:); 
    end
%end
