function output = Linv(m, input, flag, model)
% output = Linv(m, input, flag, model)
%
% Solve H*output  = input if flag=1
%       H'*output = input if flag=2
% where H = w^2*m + L
% m is the squared slowness in (s/km)^2
% w = 2*pi*model.freq
% L is the 9 point Laplacian with absorbing bc's
% The grid is defined by model.x and model.z

x = model.x; nx = length(x); dx = x(2)-x(1);
z = model.z; nz = length(z); dz = z(2)-z(1);
N = nx*nz;

nbx = ceil(.1*nx);
nbz = ceil(.1*nz);
nxt = nx + 2*nbx;
nzt = nz + 2*nbz;
Nt = nxt*nzt;
nrhs = size(input,2);

It = nbx*nzt + repmat([nbz:2*nbz:2*nx*nbz],nz,1) + reshape([1:N]',nz,nx); It = It(:);

H  = opHelm2D9pt(reshape(1e3./sqrt(m),nz,nx),x(end)-x(1),z(end)-z(1),nbx,nbz,model.freq);

tmpi = zeros(Nt,nrhs);
tmpi(It,:) = input;
if flag==1
    tmpo = H  \ tmpi;
else
    tmpo = H' \ tmpi;
end
output = tmpo(It,:);
